@@ -1,33 +1,43 @@
 # -*- coding: utf-8 -*-
 """ This module provides the backend Flask server used by psiTurk. """
+from __future__ import print_function
+from __future__ import absolute_import
 
+from .psiturk_statuses import *
+from builtins import str
+from builtins import range
 import os
 import sys
 import datetime
 import logging
 from random import choice
 import user_agents
-import string
 import requests
 import re
 import json
 
+import six
+if six.PY2:
+    # FileNotFoundError is only available since Python 3.3
+    FileNotFoundError = IOError
+    from io import open
+
 try:
     from collections import Counter
 except ImportError:
-    from counter import Counter
+    from .counter import Counter
 
 # Setup flask
 from flask import Flask, render_template, render_template_string, request, \
     jsonify
 
 # Setup database
-from db import db_session, init_db
-from models import Participant
+from .db import db_session, init_db
+from .models import Participant
 from sqlalchemy import or_, exc
 
-from psiturk_config import PsiturkConfig
-from experiment_errors import ExperimentError
+from .psiturk_config import PsiturkConfig
+from .experiment_errors import ExperimentError, InvalidUsage
 from psiturk.user_utils import nocache
 
 # Setup config
@@ -35,8 +45,11 @@ CONFIG = PsiturkConfig()
 CONFIG.load_config()
 
 # Setup logging
-LOG_FILE_PATH = os.path.join(os.getcwd(), CONFIG.get("Server Parameters", \
-    "logfile"))
+if 'ON_HEROKU' in os.environ:
+    LOG_FILE_PATH = None
+else:
+    LOG_FILE_PATH = os.path.join(os.getcwd(), CONFIG.get("Server Parameters",
+                                                         "logfile"))
 
 LOG_LEVELS = [logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR,
               logging.CRITICAL]
@@ -45,15 +58,6 @@ logging.basicConfig(filename=LOG_FILE_PATH, format='%(asctime)s %(message)s',
                     level=LOG_LEVEL)
 
 # Status codes
-NOT_ACCEPTED = 0
-ALLOCATED = 1
-STARTED = 2
-COMPLETED = 3
-SUBMITTED = 4
-CREDITED = 5
-QUITEARLY = 6
-BONUSED = 7
-
 
 # Let's start
 # ===========
@@ -64,6 +68,7 @@ app.config.update(SEND_FILE_MAX_AGE_DEFAULT=10)
 app.secret_key = CONFIG.get('Server Parameters', 'secret_key')
 app.logger.info("Secret key: " + app.secret_key)
 
+
 # Serving warm, fresh, & sweet custom, user-provided routes
 # ==========================================================
 
@@ -85,8 +90,8 @@ init_db()
 
 
 # Read psiturk.js file into memory
-PSITURK_JS_FILE = os.path.join(os.path.dirname(__file__), \
-    "psiturk_js/psiturk.js")
+PSITURK_JS_FILE = os.path.join(os.path.dirname(__file__),
+                               "psiturk_js/psiturk.js")
 app.logger.info(PSITURK_JS_FILE)
 
 if os.path.exists(PSITURK_JS_FILE):
@@ -103,6 +108,15 @@ def handle_exp_error(exception):
     return exception.error_page(request, CONFIG.get('HIT Configuration',
                                                     'contact_email_on_error'))
 
+# for use with API errors
+@app.errorhandler(InvalidUsage)
+def handle_invalid_usage(error):
+    response = jsonify(error.to_dict())
+    response.status_code = error.status_code
+    app.logger.error(error.message)
+    return response
+
+
 @app.teardown_request
 def shutdown_session(_=None):
     ''' Shut down session route '''
@@ -112,7 +126,7 @@ def shutdown_session(_=None):
 # Experiment counterbalancing code
 # ================================
 
-def get_random_condcount():
+def get_random_condcount(mode):
     """
     HITs can be in one of three states:
         - jobs that are finished
@@ -128,12 +142,19 @@ def get_random_condcount():
                                                            'cutoff_time'))
     starttime = datetime.datetime.now() + cutofftime
 
-    numconds = CONFIG.getint('Task Parameters', 'num_conds')
-    numcounts = CONFIG.getint('Task Parameters', 'num_counters')
+    try:
+        conditions = json.load(
+            open(os.path.join(app.root_path, 'conditions.json')))
+        numconds = len(list(conditions.keys()))
+        numcounts = 1
+    except IOError as e:
+        numconds = CONFIG.getint('Task Parameters', 'num_conds')
+        numcounts = CONFIG.getint('Task Parameters', 'num_counters')
 
     participants = Participant.query.\
-        filter(Participant.codeversion == \
+        filter(Participant.codeversion ==
                CONFIG.get('Task Parameters', 'experiment_code_version')).\
+        filter(Participant.mode == mode).\
         filter(or_(Participant.status == COMPLETED,
                    Participant.status == CREDITED,
                    Participant.status == SUBMITTED,
@@ -144,9 +165,11 @@ def get_random_condcount():
         for counter in range(numcounts):
             counts[(cond, counter)] = 0
     for participant in participants:
-        counts[(participant.cond, participant.counterbalance)] += 1
+        condcount = (participant.cond, participant.counterbalance)
+        if condcount in counts:
+            counts[condcount] += 1
     mincount = min(counts.values())
-    minima = [hsh for hsh, count in counts.iteritems() if count == mincount]
+    minima = [hsh for hsh, count in counts.items() if count == mincount]
     chosen = choice(minima)
     #conds += [ 0 for _ in range(1000) ]
     #conds += [ 1 for _ in range(1000) ]
@@ -170,11 +193,13 @@ def favicon():
     ''' Serve favicon '''
     return app.send_static_file('favicon.ico')
 
+
 @app.route('/static/js/psiturk.js')
 def psiturk_js():
     ''' psiTurk js route '''
     return render_template_string(PSITURK_JS_CODE)
 
+
 @app.route('/check_worker_status', methods=['GET'])
 def check_worker_status():
     ''' Check worker status route '''
@@ -183,15 +208,31 @@ def check_worker_status():
         return jsonify(**resp)
     else:
         worker_id = request.args['workerId']
-        try:
-            part = Participant.query.\
-                filter(Participant.workerid == worker_id).one()
-            status = part.status
-        except exc.SQLAlchemyError:
-            status = NOT_ACCEPTED
-        resp = {"status" : status}
+        assignment_id = request.args['assignmentId']
+        allow_repeats = CONFIG.getboolean('HIT Configuration', 'allow_repeats')
+        if allow_repeats:  # if you allow repeats focus on current worker/assignment combo
+            try:
+                part = Participant.query.\
+                    filter(Participant.workerid == worker_id).\
+                    filter(Participant.assignmentid == assignment_id).one()
+                status = part.status
+            except exc.SQLAlchemyError:
+                status = NOT_ACCEPTED
+        else:  # if you disallow repeats search for highest status of anything by this worker
+            try:
+                matches = Participant.query.\
+                    filter(Participant.workerid == worker_id).all()
+                numrecs = len(matches)
+                if numrecs == 0:  # this should be caught by exception, but just to be safe
+                    status = NOT_ACCEPTED
+                else:
+                    status = max([record.status for record in matches])
+            except exc.SQLAlchemyError:
+                status = NOT_ACCEPTED
+        resp = {"status": status}
         return jsonify(**resp)
 
+
 @app.route('/ad', methods=['GET'])
 @app.route('/pub', methods=['GET'])
 @nocache
@@ -210,8 +251,8 @@ def advertisement():
     user_agent_string = request.user_agent.string
     user_agent_obj = user_agents.parse(user_agent_string)
     browser_ok = True
-    for rule in string.split(
-            CONFIG.get('HIT Configuration', 'browser_exclude_rule'), ','):
+    browser_exclude_rule = CONFIG.get('HIT Configuration', 'browser_exclude_rule')
+    for rule in browser_exclude_rule.split(','):
         myrule = rule.strip()
         if myrule in ["mobile", "tablet", "touchcapable", "pc", "bot"]:
             if (myrule == "mobile" and user_agent_obj.is_mobile) or\
@@ -264,16 +305,21 @@ def advertisement():
     except exc.SQLAlchemyError:
         status = None
 
-    if status == STARTED and not debug_mode:
+    allow_repeats = CONFIG.getboolean('HIT Configuration', 'allow_repeats')
+    if (status == STARTED or status == QUITEARLY) and not debug_mode:
         # Once participants have finished the instructions, we do not allow
         # them to start the task again.
         raise ExperimentError('already_started_exp_mturk')
-    elif status == COMPLETED:
-        use_psiturk_ad_server = CONFIG.getboolean('Shell Parameters', 'use_psiturk_ad_server')
+    elif status == COMPLETED or (status == SUBMITTED and not already_in_db):
+        # 'or status == SUBMITTED' because we suspect that sometimes the post
+        # to mturk fails after we've set status to SUBMITTED, so really they
+        # have not successfully submitted. This gives another chance for the
+        # submit to work when not using the psiturk ad server.
+        use_psiturk_ad_server = CONFIG.getboolean(
+            'Shell Parameters', 'use_psiturk_ad_server')
         if not use_psiturk_ad_server:
-            # They've finished the experiment but haven't submitted the HIT
-            # yet.. Turn asignmentId into original assignment id before sending it
-            # back to AMT
+            # They've finished the experiment but haven't successfully submitted the HIT
+            # yet.
             return render_template(
                 'thanks-mturksubmit.html',
                 using_sandbox=(mode == "sandbox"),
@@ -281,15 +327,15 @@ def advertisement():
                 assignmentid=assignment_id,
                 workerid=worker_id
             )
-        else: 
+        else:
             # Show them a thanks message and tell them to go away.
-            return render_template( 'thanks.html' )
-    elif already_in_db and not debug_mode:
+            return render_template('thanks.html')
+    elif already_in_db and not (debug_mode or allow_repeats):
         raise ExperimentError('already_did_exp_hit')
     elif status == ALLOCATED or not status or debug_mode:
         # Participant has not yet agreed to the consent. They might not
         # even have accepted the HIT.
-        with open('templates/ad.html', 'r') as temp_file:
+        with open('templates/ad.html', 'r', encoding='utf-8') as temp_file:
             ad_string = temp_file.read()
         ad_string = insert_mode(ad_string, mode)
         return render_template_string(
@@ -301,6 +347,7 @@ def advertisement():
     else:
         raise ExperimentError('status_incorrectly_set')
 
+
 @app.route('/consent', methods=['GET'])
 @nocache
 def give_consent():
@@ -314,7 +361,7 @@ def give_consent():
     assignment_id = request.args['assignmentId']
     worker_id = request.args['workerId']
     mode = request.args['mode']
-    with open('templates/consent.html', 'r') as temp_file:
+    with open('templates/consent.html', 'r', encoding='utf-8') as temp_file:
         consent_string = temp_file.read()
     consent_string = insert_mode(consent_string, mode)
     return render_template_string(
@@ -324,6 +371,7 @@ def give_consent():
         workerid=worker_id
     )
 
+
 def get_ad_via_hitid(hit_id):
     ''' Get ad via HIT id '''
     username = CONFIG.get('psiTurk Access', 'psiturk_access_key_id')
@@ -339,6 +387,7 @@ def get_ad_via_hitid(hit_id):
         else:
             return "error"
 
+
 @app.route('/exp', methods=['GET'])
 @nocache
 def start_exp():
@@ -351,7 +400,7 @@ def start_exp():
     worker_id = request.args['workerId']
     mode = request.args['mode']
     app.logger.info("Accessing /exp: %(h)s %(a)s %(w)s " % {
-        "h" : hit_id,
+        "h": hit_id,
         "a": assignment_id,
         "w": worker_id
     })
@@ -362,13 +411,21 @@ def start_exp():
 
     # Check first to see if this hitId or assignmentId exists.  If so, check to
     # see if inExp is set
-    matches = Participant.query.\
-        filter(Participant.workerid == worker_id).\
-        all()
+    allow_repeats = CONFIG.getboolean('HIT Configuration', 'allow_repeats')
+    if allow_repeats:
+        matches = Participant.query.\
+            filter(Participant.workerid == worker_id).\
+            filter(Participant.assignmentid == assignment_id).\
+            all()
+    else:
+        matches = Participant.query.\
+            filter(Participant.workerid == worker_id).\
+            all()
+
     numrecs = len(matches)
     if numrecs == 0:
         # Choose condition and counterbalance
-        subj_cond, subj_counter = get_random_condcount()
+        subj_cond, subj_counter = get_random_condcount(mode)
 
         worker_ip = "UNKNOWN" if not request.remote_addr else \
             request.remote_addr
@@ -389,7 +446,8 @@ def start_exp():
             ipaddress=worker_ip,
             browser=browser,
             platform=platform,
-            language=language
+            language=language,
+            mode=mode
         )
         part = Participant(**participant_attributes)
         db_session.add(part)
@@ -426,7 +484,8 @@ def start_exp():
             if other_assignment:
                 raise ExperimentError('already_did_exp_hit')
 
-    use_psiturk_ad_server = CONFIG.getboolean('Shell Parameters', 'use_psiturk_ad_server')
+    use_psiturk_ad_server = CONFIG.getboolean(
+        'Shell Parameters', 'use_psiturk_ad_server')
     if use_psiturk_ad_server and (mode == 'sandbox' or mode == 'live'):
         # If everything goes ok here relatively safe to assume we can lookup
         # the ad.
@@ -437,7 +496,7 @@ def start_exp():
                     + str(ad_id)
             elif mode == "live":
                 ad_server_location = 'https://ad.psiturk.org/complete/' +\
-                str(ad_id)
+                    str(ad_id)
         else:
             raise ExperimentError('hit_not_registered_with_ad_server')
     else:
@@ -448,9 +507,12 @@ def start_exp():
         condition=part.cond,
         counterbalance=part.counterbalance,
         adServerLoc=ad_server_location,
-        mode = mode
+        mode=mode,
+        contact_address=CONFIG.get(
+            'HIT Configuration', 'contact_email_on_error')
     )
 
+
 @app.route('/inexp', methods=['POST'])
 def enterexp():
     """
@@ -510,6 +572,7 @@ def load(uid=None):
 
     return jsonify(**resp)
 
+
 @app.route('/sync/<uid>', methods=['PUT'])
 def update(uid=None):
     """
@@ -526,22 +589,20 @@ def update(uid=None):
         app.logger.error("DB error: Unique user not found.")
 
     if hasattr(request, 'json'):
-        user.datastring = request.data.decode('utf-8').encode(
-            'ascii', 'xmlcharrefreplace'
-        )
+        user.datastring = json.dumps(request.json)
         db_session.add(user)
         db_session.commit()
 
-    try:
-        data = json.loads(user.datastring)
-    except:
-        data = {}
+    
+    data = json.loads(user.datastring)
+    
 
     trial = data.get("currenttrial", None)
     app.logger.info("saved data for %s (current trial: %s)", uid, trial)
     resp = {"status": "user data saved"}
     return jsonify(**resp)
 
+
 @app.route('/quitter', methods=['POST'])
 def quitter():
     """
@@ -582,10 +643,7 @@ def debug_complete():
         raise ExperimentError('improper_inputs')
     else:
         unique_id = request.args['uniqueId']
-        if unique_id[:5] == "debug":
-            debug_mode = True
-        else:
-            debug_mode = False
+        mode = request.args['mode']
         try:
             user = Participant.query.\
                 filter(Participant.uniqueid == unique_id).one()
@@ -596,10 +654,12 @@ def debug_complete():
         except:
             raise ExperimentError('error_setting_worker_complete')
         else:
-            if debug_mode:
-                return render_template('complete.html')
-            else: # send them back to mturk.
+            # send them back to mturk.
+            if (mode == 'sandbox' or mode == 'live'):
                 return render_template('closepopup.html')
+            else:
+                return render_template('complete.html')
+
 
 @app.route('/worker_complete', methods=['GET'])
 def worker_complete():
@@ -620,9 +680,10 @@ def worker_complete():
             status = "success"
         except exc.SQLAlchemyError:
             status = "database error"
-        resp = {"status" : status}
+        resp = {"status": status}
         return jsonify(**resp)
 
+
 @app.route('/worker_submitted', methods=['GET'])
 def worker_submitted():
     ''' Submit worker '''
@@ -641,7 +702,7 @@ def worker_submitted():
             status = "success"
         except exc.SQLAlchemyError:
             status = "database error"
-        resp = {"status" : status}
+        resp = {"status": status}
         return jsonify(**resp)
 
 # Is this a security risk?
@@ -653,9 +714,11 @@ def ppid():
 
 # Insert "mode" into pages so it's carried from page to page done server-side
 # to avoid breaking backwards compatibility with old templates.
+
+
 def insert_mode(page_html, mode):
     ''' Insert mode '''
-    page_html = page_html.decode("utf-8")
+    page_html = page_html
     match_found = False
     matches = re.finditer('workerId={{ workerid }}', page_html)
     match = None
@@ -685,12 +748,16 @@ def regularpage(foldername=None, pagename=None):
     else:
         return render_template(foldername+"/"+pagename)
 
+
 def run_webserver():
     ''' Run web server '''
     host = "0.0.0.0"
     port = CONFIG.getint('Server Parameters', 'port')
-    print "Serving on ", "http://" +  host + ":" + str(port)
+    print("Serving on ", "http://" + host + ":" + str(port))
+    app.config['TEMPLATES_AUTO_RELOAD'] = True
+    app.jinja_env.auto_reload = True
     app.run(debug=True, host=host, port=port)
 
+
 if __name__ == '__main__':
     run_webserver()
