@@ -1,13 +1,18 @@
+from __future__ import print_function
+from __future__ import absolute_import
 # myapp.mycustomapplication
+from builtins import str
 from gunicorn.app.base import Application
 from gunicorn import util
 import multiprocessing
-from psiturk_config import PsiturkConfig
+from psiturk.psiturk_config import PsiturkConfig
 import os
+import hashlib
 
 config = PsiturkConfig()
 config.load_config()
 
+
 class ExperimentServer(Application):
     '''
     Custom Gunicorn Server Application that serves up the Experiment application
@@ -23,11 +28,7 @@ class ExperimentServer(Application):
         self.options = self.user_options
         self.prog = None
         self.do_load_config()
-        if 'OPENSHIFT_SECRET_TOKEN' in os.environ:
-            my_ip = os.environ['OPENSHIFT_APP_DNS']
-            print "Now serving on " + os.environ['OPENSHIFT_APP_DNS']
-        else:
-            print "Now serving on", "http://" + self.options["bind"]
+        print("Now serving on", "http://" + self.options["bind"])
 
     def init(self, *args):
         '''init method
@@ -35,7 +36,7 @@ class ExperimentServer(Application):
         dict which specifies custom settings.
         '''
         cfg = {}
-        for k, v in self.options.items():
+        for k, v in list(self.options.items()):
             if k.lower() in self.cfg.settings and v is not None:
                 cfg[k.lower()] = v
         return cfg
@@ -47,7 +48,8 @@ class ExperimentServer(Application):
         return util.import_app("psiturk.experiment:app")
 
     def load_user_config(self):
-        workers = config.get("Server Parameters", "threads")  # config calls these threads to avoid confusing with workers
+        # config calls these threads to avoid confusing with workers
+        workers = config.get("Server Parameters", "threads")
         if workers == "auto":
             workers = str(multiprocessing.cpu_count() * 2 + 1)
 
@@ -61,9 +63,11 @@ class ExperimentServer(Application):
             has shut down until they hit `enter` and see that 
             the cmdloop prompt suddenly says "server off"
             '''
-            print 'Caught ^C, experiment server has shut down.'
-            print 'Press `enter` to continue.'
+            print('Caught ^C, experiment server has shut down.')
+            print('Press `enter` to continue.')
 
+        # add unique identifier of this psiturk project folder
+        project_hash = hashlib.sha1(os.getcwd().encode()).hexdigest()[:12]
         self.user_options = {
             'bind': config.get("Server Parameters", "host") + ":" + config.get("Server Parameters", "port"),
             'workers': workers,
@@ -71,22 +75,33 @@ class ExperimentServer(Application):
             'loglevel': self.loglevels[config.getint("Server Parameters", "loglevel")],
             # 'accesslog': config.get("Server Parameters", "logfile"),
             'errorlog': config.get("Server Parameters", "logfile"),
-            'proc_name': 'psiturk_experiment_server',
+            'proc_name': 'psiturk_experiment_server_' + project_hash,
             'limit_request_line': '0',
             'on_exit': on_exit
         }
 
         if config.has_option("Server Parameters", "certfile") and config.has_option("Server Parameters", "keyfile"):
-            print "Loading SSL certs for server..."
+            print("Loading SSL certs for server...")
             ssl_options = {
-                'certfile' : config.get("Server Parameters", "certfile"),
-                'keyfile' : config.get("Server Parameters", "keyfile")
+                'certfile': config.get("Server Parameters", "certfile"),
+                'keyfile': config.get("Server Parameters", "keyfile")
             }
             self.user_options.update(ssl_options)
 
+        if config.has_option("Server Parameters", "server_timeout"):
+            self.user_options.update(
+                {'timeout': config.get("Server Parameters", "server_timeout")})
+
+        if 'ON_HEROKU' in os.environ:
+            self.user_options.update({
+                'accesslog': '-',
+                'errorlog': '-'
+            })
+
 
 def launch():
     ExperimentServer().run()
 
+
 if __name__ == "__main__":
     launch()
